# Worst case analysis

from definitions import *
from pyopus.design.wc import WorstCase
# If MPI is imported an application not using MPI will behave correctly
# (i.e. only slot 0 will run the program) even when started with mpirun/mpiexec
from pyopus.parallel.mpi import MPI
from pyopus.parallel.cooperative import cOS


if __name__=='__main__':
	atDesign={
		# Result of design across corners
		'c_out':    6.4025057781114802e-12,
		'diff_l':   3.4247816773735976e-07,
		'diff_w':   1.6738092946223754e-05,
		'load_l':   2.3668072815514710e-06,
		'load_w':   5.3240803355989389e-05,
		'mirr_l':   1.0157035640188838e-06,
		'mirr_ld':  1.6182520447950098e-06,
		'mirr_w':   5.8107861163445226e-05,
		'mirr_wd':  6.1242813846216839e-05,
		'mirr_wo':  6.8277846319352392e-05,
		'out_l':    4.1088626829554061e-07,
		'out_w':    8.1812469568552345e-05,
		'r_out':    5.1647907484619873e+02,
	}
	
	# Create corner with the Monte Carlo model, defined for all heads
	corners={
		'mc': {
			'params': {}, 
			'modules': [ 'mc' ]
		},
	}
	
	# Worst case performances (skip area)
	wcList=list(measures.keys())
	wcList.sort()
	wcList.remove('area')
	
	# Prepare parallel environment
	cOS.setVM(MPI(mirrorMap={'*':'.'}, debug=0))
	
	# 3-sigma worst case
	wc=WorstCase(
		heads, analyses, measures, corners, statParams, opParams, variables=variables, 
		fixedParams=atDesign, 
		beta=3.0, sigmaBox=10, 
		stepTol=0.01, 
		debug=4, 
		screenOptions={'debug': 1}, 
		spawnerLevel=0
	)
	wcresults, analysisCount = wc(wcList)
	# wcresults, analysisCount = wc(['vgs_drv'])
	# wcresults, analysisCount = wc(['cmrr', 'vgs_drv'])
	# wcresults, analysisCount = wc(['cmrr'])
	
	print(wc.formatResults())
	print(wc.analysisCount)
	
	# Finalize cOS parallel environment
	cOS.finalize()

# Result	
#       cmrr >     5.83505e+01 o goal=   6.00000e+01 d=    2.99443e+00 neval= 261 3 OK
#       gain >     6.54893e+01   goal=   6.00000e+01 d=    3.00000e+00 neval= 165 2 OK
#       isup <     3.09901e-04   goal=   1.00000e-03 d=    3.00000e+00 neval= 115 1 OK
#     out_op >    -7.85431e-03   goal=  -1.25000e-02 d=    3.00000e+00 neval= 164 1 OK
#     out_op <     1.89868e-02 o goal=   1.25000e-02 d=    3.00000e+00 neval= 120 1 OK
#   overshdn <     1.02256e-01   goal=   1.50000e-01 d=    3.00000e+00 neval= 203 2 OK
#   overshup <     1.34256e-01   goal=   1.50000e-01 d=    2.98490e+00 neval= 186 1 OK
#   ph_slope <    -0.00000e+00   goal=   0.00000e+00 d=    0.00000e+00 neval= 199 1 OK
#         pm >     6.62313e+01   goal=   5.00000e+01 d=    3.00000e+00 neval= 116 1 OK
#   psrr_vdd >     6.91351e+01   goal=   6.00000e+01 d=    2.99716e+00 neval= 330 3 OK
#   psrr_vss >     7.19386e+01   goal=   6.00000e+01 d=    2.85396e+00 neval= 370 3 OK
#     slewdn >     1.10018e+07   goal=   2.00000e+06 d=    3.00000e+00 neval= 130 1 OK
#     slewup >     1.06832e+07   goal=   2.00000e+06 d=    3.00000e+00 neval= 137 1 OK
#      swing >     1.30006e+00   goal=   1.00000e+00 d=    3.00000e+00 neval= 115 1 OK
#     tsetdn <     1.64895e-07   goal=   1.00000e-06 d=    3.00000e+00 neval= 183 2 OK
#     tsetup <     1.76100e-07   goal=   1.00000e-06 d=    3.00000e+00 neval= 116 1 OK
#       ugbw >     1.15858e+07   goal=   1.00000e+07 d=    3.00000e+00 neval= 175 2 OK
# vds_drv[0] >     5.19811e-01   goal=   0.00000e+00 d=    3.00000e+00 neval= 119 1 OK
# vds_drv[1] >     6.80890e-01   goal=   0.00000e+00 d=    3.00000e+00 neval= 124 1 OK
# vds_drv[2] >     2.47832e-01   goal=   0.00000e+00 d=    3.00000e+00 neval= 130 1 OK
# vds_drv[3] >     4.44189e-02   goal=   0.00000e+00 d=    3.00000e+00 neval= 111 1 OK
# vds_drv[4] >     7.07933e-01   goal=   0.00000e+00 d=    2.99685e+00 neval= 391 4 OK
# vds_drv[5] >     3.44992e-01   goal=   0.00000e+00 d=    3.00000e+00 neval= 112 1 OK
# vds_drv[6] >     1.90274e-01   goal=   0.00000e+00 d=    3.00000e+00 neval= 108 1 OK
# vds_drv[7] >     7.25887e-01   goal=   0.00000e+00 d=    2.99705e+00 neval= 211 2 OK
# vgs_drv[0] >     1.10325e-02   goal=   0.00000e+00 d=    3.00000e+00 neval= 171 2 OK
# vgs_drv[1] >     8.60513e-03   goal=   0.00000e+00 d=    3.00000e+00 neval= 114 1 OK
# vgs_drv[2] >     7.13397e-02   goal=   0.00000e+00 d=    3.00000e+00 neval= 143 1 OK
# vgs_drv[3] >     8.85646e-02   goal=   0.00000e+00 d=    3.00000e+00 neval= 194 1 OK
# vgs_drv[4] >     7.11217e-02   goal=   0.00000e+00 d=    3.00000e+00 neval= 132 1 OK
# vgs_drv[5] >     2.31932e-01   goal=   0.00000e+00 d=    2.97945e+00 neval= 459 3 OK
# vgs_drv[6] >     2.31489e-01   goal=   0.00000e+00 d=    3.00000e+00 neval= 348 2 OK
# vgs_drv[7] >     6.69440e-02   goal=   0.00000e+00 d=    3.00000e+00 neval= 110 1 OK
