# Worst case distance

from definitions import *
from pyopus.design.wcd import WorstCaseDistance
from pyopus.design.wc import WorstCase
# If MPI is imported an application not using MPI will behave correctly
# (i.e. only slot 0 will run the program) even when started with mpirun/mpiexec
from pyopus.parallel.mpi import MPI
from pyopus.parallel.cooperative import cOS


if __name__=='__main__':
	atDesign={
		# Result of yield targeting starting from result of design across corners
		'c_out':    1.5906247662071053e-12,
		'diff_l':   1.0647898762052580e-06,
		'diff_w':   1.0954427560389812e-05,
		'load_l':   1.9692895593955935e-06,
		'load_w':   1.8559908362865259e-05,
		'mirr_l':   9.7786062346348561e-07,
		'mirr_ld':  2.8778986167864697e-06,
		'mirr_w':   2.2130751573351460e-05,
		'mirr_wd':  9.8183046192244794e-06,
		'mirr_wo':  6.0483504290617440e-05,
		'out_l':    5.8015162233222138e-07,
		'out_w':    3.5473803134446091e-05,
		'r_out':    2.1456835044861632e+03,
	}
	
	# Create corner with the Monte Carlo model
	corners={
		'mc': {
			'params': {}, 
			'modules': [ 'mc' ]
		}
	}
		
	# Worst case performances (skip area, phase slope)
	wcList=list(measures.keys())
	wcList.sort()
	wcList.remove('area')
	wcList.remove('ph_slope')
		
	# Prepare parallel environment
	cOS.setVM(MPI(mirrorMap={'*':'.'}))
	
	# 3-sigma worst case
	wc=WorstCaseDistance(
		heads, analyses, measures, corners, statParams, opParams, variables=variables, 
		fixedParams=atDesign, 
		stepTol=0.01,  # Try 0.0001
		sigmaBox=10, 
		debug=2, 
		spawnerLevel=0
	)
		
	wcresults, analysisCount = wc(wcList)
	# wcresults, analysisCount = wc(['vgs_drv'])
	# wcresults, analysisCount = wc(['cmrr', 'vgs_drv'])
	# wcresults, analysisCount = wc(['cmrr'])
	# wcresults, analysisCount = wc(['ugbw'])
	# wcresults, analysisCount = wc(['vgs_drv'])
	
	print(wc.formatResults())
	print(wc.analysisCount)
	
	# Finalize cOS parallel environment
	cOS.finalize()

# Result of yield targeting (WCD)
#       cmrr >     6.00000e+01 wc=    6.00007e+01 wcd=    4.68316e+00 neval= 214 2 OK
#       gain >     6.00000e+01 wc=    5.98489e+01 wcd=    9.97291e+00 neval= 723 2 OK
#       isup <     1.00000e-03 wc=    4.75340e-04 wcd=    2.19977e+01 neval= 285 1 SENS+
#     out_op >    -1.25000e-02 wc=   -1.25000e-02 wcd=    3.07258e+00 neval= 128 1 OK
#     out_op <     1.25000e-02 wc=    1.25009e-02 wcd=    4.46307e+00 neval= 113 1 OK
#   overshdn <     1.50000e-01 wc=    1.50265e-01 wcd=    1.08847e+01 neval=1599 3 OK
#   overshup <     1.50000e-01 wc=    1.49931e-01 wcd=    4.31007e+00 neval= 306 3 OK
#         pm >     5.00000e+01 wc=    6.70226e+01 wcd=    1.08984e+01 neval= 163 1 OUTSIDE+
#   psrr_vdd >     6.00000e+01 wc=    6.00519e+01 wcd=    6.44687e+00 neval= 698 2 OK
#   psrr_vss >     6.00000e+01 wc=    6.30659e+01 wcd=    2.36071e+01 neval= 368 2 OUTSIDE+
#     slewdn >     2.00000e+06 wc=    7.09692e+06 wcd=    3.15511e+01 neval= 245 1 SENS+
#     slewup >     2.00000e+06 wc=    2.92584e+06 wcd=    2.62814e+01 neval= 317 2 OUTSIDE+
#      swing >     1.00000e+00 wc=    1.00000e+00 wcd=    7.25072e+00 neval= 131 1 OK
#     tsetdn <     1.00000e-06 wc=    2.70461e-07 wcd=    2.61711e+01 neval= 170 1 SENS+
#     tsetup <     1.00000e-06 wc=    3.65365e-07 wcd=    2.18296e+01 neval= 398 3 OUTSIDE+
#       ugbw >     1.00000e+07 wc=    1.00000e+07 wcd=    5.84030e+00 neval= 129 1 OK
# vds_drv[0] >     0.00000e+00 wc=    9.92140e-02 wcd=    1.46385e+01 neval= 131 1 OUTSIDE+
# vds_drv[1] >     0.00000e+00 wc=    4.81745e-09 wcd=    1.17263e+01 neval= 123 1 OK
# vds_drv[2] >     0.00000e+00 wc=    9.20813e-02 wcd=    1.00000e+01 neval= 116 1 OUTSIDE+
# vds_drv[3] >     0.00000e+00 wc=    1.13889e-07 wcd=    3.77566e+00 neval= 119 1 OK
# vds_drv[4] >     0.00000e+00 wc=    2.53391e-02 wcd=    2.66602e+01 neval= 265 1 SENS+
# vds_drv[5] >     0.00000e+00 wc=    8.47352e-02 wcd=    1.00000e+01 neval= 111 1 OUTSIDE+
# vds_drv[6] >     0.00000e+00 wc=    1.58053e-01 wcd=    1.41421e+01 neval= 116 1 OUTSIDE+
# vds_drv[7] >     0.00000e+00 wc=    3.62863e-01 wcd=    2.56340e+01 neval= 275 1 SENS+
# vgs_drv[0] >     0.00000e+00 wc=    1.40994e-02 wcd=    1.92609e+01 neval= 277 3 OUTSIDE+
# vgs_drv[1] >     0.00000e+00 wc=    1.18576e-03 wcd=    2.23437e+01 neval= 208 1 OUTSIDE+
# vgs_drv[2] >     0.00000e+00 wc=    1.19259e-01 wcd=    1.31447e+01 neval= 147 1 OUTSIDE+
# vgs_drv[3] >     0.00000e+00 wc=    1.36934e-01 wcd=    2.00000e+01 neval= 142 1 OUTSIDE+
# vgs_drv[4] >     0.00000e+00 wc=    1.07195e-01 wcd=    2.00000e+01 neval= 206 1 OUTSIDE+
# vgs_drv[5] >     0.00000e+00 wc=    1.33427e-01 wcd=    2.42561e+01 neval= 342 3 OUTSIDE+
# vgs_drv[6] >     0.00000e+00 wc=    1.46258e-01 wcd=    1.00013e+01 neval= 165 2 SENS+
# vgs_drv[7] >     0.00000e+00 wc=    1.79045e-01 wcd=    1.41498e+01 neval= 194 2 OUTSIDE+
